/**
 * @file jvmDefines.h
 */
#ifndef JVM_DEFINES_H
#define JVM_DEFINES_H

#define FALSE 0
#define TRUE -1

#define false FALSE
#define true TRUE


// Javelin IDE constants
#define    kDebugMagic        0x1D
#define    kRunMagic          0xA4

//activity values
#define    kPassive 0   //JVM was running but has stopped (exception,end)
#define    kRun     1   //JVM running
#define    kJog     2   //???
#define    kBreak   3   //JVM reached breakpoint
#define    kEnd     4   //java program ended

#define    nmPortA       0x06  //corresponds to I/O pins 0..7
#define    nmPortB       0x07  //corresponds to I/O pins 8..15
//reserved values for extending Javelin code to PropJava code
#define    nmPortC       0x08  //corresponds to I/O pins 16..23
#define    nmPortD       0x09  //corresponds to I/O pins 24..31
  
#define    pin0  (1<<0)         //A constant representing pin P0
#define    pin1  (1<<1)         //A constant representing pin P1
#define    pin2  (1<<2)         //A constant representing pin P2
#define    pin3  (1<<3)         //A constant representing pin P3 
#define    pin4  (1<<4)         //A constant representing pin P4 
#define    pin5  (1<<5)         //A constant representing pin P5
#define    pin6  (1<<6)         //A constant representing pin P6 
#define    pin7  (1<<7)         //A constant representing pin P7 
#define    pin8  ((1<<0) + 0x100)  //A constant representing pin P8
#define    pin9  ((1<<1) + 0x100)  //A constant representing pin P9
#define    pin10 ((1<<2) + 0x100)  //A constant representing pin P10
#define    pin11 ((1<<3) + 0x100)  //A constant representing pin P11 
#define    pin12 ((1<<4) + 0x100)  //A constant representing pin P12 
#define    pin13 ((1<<5) + 0x100)  //A constant representing pin P13
#define    pin14 ((1<<6) + 0x100)  //A constant representing pin P14
#define    pin15 ((1<<7) + 0x100)  //A constant representing pin P15  
//reserved values for extending Javelin code to PropJava code
#define    pin16  ((1<<0) + 0x200)  //A constant representing pin P16
#define    pin17  ((1<<1) + 0x200)  //A constant representing pin P17
#define    pin18  ((1<<2) + 0x200)  //A constant representing pin P18
#define    pin19  ((1<<3) + 0x200)  //A constant representing pin P19 
#define    pin20  ((1<<4) + 0x200)  //A constant representing pin P20 
#define    pin21  ((1<<5) + 0x200)  //A constant representing pin P21
#define    pin22  ((1<<6) + 0x200)  //A constant representing pin P22
#define    pin23  ((1<<7) + 0x200)  //A constant representing pin P23  
#define    pin24  ((1<<0) + 0x300)  //A constant representing pin P24
#define    pin25  ((1<<1) + 0x300)  //A constant representing pin P25
#define    pin26  ((1<<2) + 0x300)  //A constant representing pin P26
#define    pin27  ((1<<3) + 0x300)  //A constant representing pin P27 
#define    pin28  ((1<<4) + 0x300)  //A constant representing pin P28 
#define    pin29  ((1<<5) + 0x300)  //A constant representing pin P29
#define    pin30  ((1<<6) + 0x300)  //A constant representing pin P30
#define    pin31  ((1<<7) + 0x300)  //A constant representing pin P31  

//native registers

//file stamp\core\EEPROM.java
#define    JVM_SB_MSB  0x4C  // Highbyte of stackbase (first free eeprom address)
#define    JVM_SB_LSB  0x4D  // Lowbyte of stackbase (first free eeprom address)
  /*
     The EEPROM.java methods size(),write() and read() use 0x7FFF for eeprom size.
     On the prop the eeprom size is set to JAVASIZE. To let the methods access the
     correct addresses, we must adjust for this difference. That means the value
     from register JVM_SB must be increased by JVM_SB_ADJUST. Originally JVM_SB is
     set to the stack base as found on eeprom address 0x0012.
  */

//file stamp\core\Memory.java
#define    JVM_HEAP_POINTER_LSB   0x18   //jvm heap pointer (word)
#define    JVM_HEAP_POINTER_MSB   0x19
#define    JVM_STACK_POINTER_LSB  0x12   //jvm stack pointer (word)
#define    JVM_STACK_POINTER_MSB  0x13

  /*
         Stackframe format
         +---------------+
         | Local0        | <- JVM_FP
         | ...           |
         | LocalN        |
         +---------------+
         | Method MSB    | <- JVM_FP + numLocals
         | Method LSB    |
         +---------------+
         | Old FP MSB    |
         | Old FP LSB    |
         +---------------+
         | JVM_PC MSB    |
         | JVM_PC LSB    |
         +---------------+
         | Stack0        |
         | ....          |
         +---------------+
                           <- JVM_SP
  */

#define    kMPOffset  0   //method pointer offset
#define    kFPOffset  2   //frame pointer offset
#define    kPCOffset  4   //program counter offset

  /*
         Memory map layout
  0x0000  +---------------+ @javaProg
         |    header     |
  0x0014  +---------------+
         | jemcodes      |
         | ...           |
  0xXXX0  +---------------+ stackbase                 ---
         | Local0        | <- JVM_FP                  ^
         | ...           |                            |
         | LocalN        |                            |
         +---------------+                            |
         | Method MSB    | <- JVM_FP + numLocals      |
         | Method LSB    |                            |
         +---------------+                            |
         | Old FP MSB    |                            |
         | Old FP LSB    |
         +---------------+                       stack frame
         | JVM_PC MSB    |                            
         | JVM_PC LSB    |                            | 
         +---------------+                            |
         | Stack0        | operand stack              |
         | ....          |                            V
         +---------------+                           ---
         |               | <- JVM_SP
         |     free      |
         |               |

         |               |
         |     free      |
         |               |
         +---------------+
         |               | <- JVM_HP
         |   allocated   |
         |               |
         +---------------+
         |  static vars  | size := byte[@javaProg + JVM_STATIC]   'size in bytes
         |  each var     | if size == 0 
         |    2 bytes    |   size := 256
         +---------------+                           
  JAVASIZE                 heapbase (on javelin 0x7FFF)

  Header layout:
  
  0000:   dw      0xA4     ; Placeholder for run/debug.
  0001:   dw      0x0      ; Placeholder for the length.
  0002:   dw      0x01     ; Version: 0.1
  0003:   dw      16      ; Number of static fields = 8 (maximum of 128 static fields)
  0004:   dw      0x00     ; `main' method: Offset hi = 0x00
  0005:   dw      0x2C     ;                       lo = 0x2C
  0006:   dw      0x00     ; java.lang.Object class: Offset hi = 0x00
  0007:   dw      0x8D     ;                                lo = 0x8D
  0008:   dw      0x01     ; java.lang.String class: Offset hi = 0x01
  0009:   dw      0x59     ;                                lo = 0x59
  000A:   dw      0x00     ; Throwable.throwVMException method: Offset hi = 0x00
  000B:   dw      0x52     ;                                           lo = 0x52
  000C:   dw      0x09     ; String table offset hi = 0x09
  000D:   dw      0x7F     ;                     lo = 0x7F
  000E:   dw      0x00     ; Callback method offset hi = 0x00
  000F:   dw      0x00     ;                        lo = 0x00
  0010:   dw      0x01     ; java.lang.OutOfMemoryError class: Offset hi = 0x01
  0011:   dw      0x2C     ;                                          lo = 0x2C
  0012:   dw      0x09     ; Stack base hi = 0x09
  0013:   dw      0xB0     ;            lo = 0xB0
  
  */

//javaProg header fields
#define    JVM_RUNDEBUG      0x0000
#define    JVM_LENGTH        0x0001
#define    JVM_VERSION       0x0002
#define    JVM_STATIC        0x0003
#define    JVM_MAIN          0x0004
#define    JVM_OBJECT        0x0006
#define    JVM_STRING        0x0008
#define    JVM_THROW         0x000A
#define    JVM_STRTABLE      0x000C
#define    JVM_CALLBACK      0x000E
#define    JVM_MEMERROR      0x0010
#define    JVM_STKBASE       0x0012

//class header fields
#define    JVM_SUPER         0x00
#define    JVM_FIELDS        0x02
  
//method header fields
#define    JVM_PARAMETERS    0x00
#define    JVM_LOCALS        0x01
#define    JVM_FLAGS         0x02
#define    JVM_CODE          0x03
#define    JVM_EXCEPTION     0x05
  
#define    BREAKPOINTS  10     //number of breakpoints supported by debugger

 
//Javelin IDE constants
#define  kReplyBit           0x80
#define  kFlag               0x7E
#define  kEscape             0x7D
#define  kInterrupt          0x19
#define  kResetWait          10000 // Time taken for SX to reset.
#define  kAckWait            5000  // The amount of time (in milliseconds) to wait for acknowledgement.
#define  kResetCharWait      3     // The amount of time to wait before sending first char after a reset.
#define  kProgramWait        2000  // The amount of time (in milliseconds) to wait for acknowledgement when programming.
#define  kStepWait           20000 // The amount of time (in milliseconds) to wait for acknowledgement when stepping.
#define  kBaseVersion        0x50  // Compatible base version of the firmware.
#define  kMaxQueueLength     500   // Maximum number of characters than can be queued.
#define  kComNone            0
#define  kComStatus          1
#define  kComReset           2
#define  kComStep            3
#define  kComRun             4
#define  kComError           5
#define  kComStop            6
#define  kComStack           7
#define  kComConfig          8
#define  kComHeap            9
#define  kComBreakpoints     10
#define  kComProgram         11
#define  kComDownloadOK      12
#define  kComStepDone        13
#define  kComAtn             14
#define  kComStartLoad       15
//#define  kComEndLoad         kBaseVersion
#define  kComTerminalQuery   17     // Check for data from the terminal.
#define  kComTerminalData    18     // Data follows.
#define  kComTerminalNoData  19     // No data follows.
#define  kComDownloadRamOK   20

//values following kComError
#define  kErrNone        0
#define  kErrOpcode      1
#define  kErrBreakpoint  2
#define  kErrMessage     3   //to send message to IDE message window
#define  kErrException   4   //to send exception to IDE status window
#define  kErrEnd         5   //to signal end of java program
#define  kErrJvm         6   //to send message to IDE status window


//Spin stamp debug pin assignments
#define  stampSOUT  16 //serial out (pin 1 of spin stamp)
#define  stampSIN   17 //serial in  (pin 2 of spin stamp)
#define  stampATN   18 //digital in (pin 3 of spin stamp, when activated, do reboot)

//Propeller system pin assignments
#define  propSCL  28 //external eeprom SCL
#define  propSDA  29 //external eeprom SDA
#define  propTX   30 //programming output
#define  propRX   31 //programming input


/*
*****************************************
* File jvmDefines: constant definitions *
*****************************************
*/


//Virtual Peripheral types
#define  NONE   0
#define  TIMER  1
#define  ADC    2
#define  DAC    3
#define  PWM    4
#define  UARTRX 5
#define  UARTTX 6

#define  MAX_NUM_VPS  6
  
// The SX banks the VPs use.
#define  nmPeriph0Bank  0x80
#define  nmPeriph1Bank  (nmPeriph0Bank + 0x10)
#define  nmPeriph2Bank  (nmPeriph0Bank + 0x20)
#define  nmPeriph3Bank  (nmPeriph0Bank + 0x30)
#define  nmPeriph4Bank  (nmPeriph0Bank + 0x40)
#define  nmPeriph5Bank  (nmPeriph0Bank + 0x50)

//vector register (offsets within a VP bank) 
#define  nmPeriphISRVec  0x00  //each bank, holds VP type

//propeller fields for pinmasks in vpRambank slots
#define  nmInmask  16    //input pin 
#define  nmOutmask  20   //output pin
  
//file stamp\core\Timer.java (offsets within a VP bank)
#define  nmLatchTimer1   0x06  //32bit latch register
#define  nmLatchTimer2   0x07
#define  nmLatchTimer3   0x08
#define  nmLatchTimer4   0x09  
#define  nmTimer1        0x02  //32bit free running timer, incremented every 8.68 microsecond
#define  nmTimer2        0x03
#define  nmTimer3        0x04
#define  nmTimer4        0x05

//file stamp\core\ADC.java (offsets within a VP bank) 
#define  nmAdcValue  0x0B  // Current ADC value.
#define  nmInPort    0x0C  // 0 for PortA, 1 for PortB (reserved: 2 for PortC, 3 for PortD)
#define  nmInPin     0x0D  // bitmask (1 of 8 set)
#define  nmOutPort   0x0E  // 0 for PortA, 1 for PortB (reserved: 2 for PortC, 3 for PortD)
#define  nmOutPin    0x0F  // bitmask (1 of 8 set)
//unused registers used for accumulator, count and pin masks
#define  nmAdcAccu     0x0A
#define  nmAdcCount    0x01
//nmAdcInMask   0x02  //long-aligned input pin mask
//nmAdcOutMask  0x06  //long-aligned output pin mask
  
//file stamp\core\DAC.java (offsets within a VP bank) 
#define  nmDacValue  0x0B  // Current DAC value (dutycycle).
#define  nmDacPort   0x0E  // 0 for PortA, 1 for PortB (reserved: 2 for PortC, 3 for PortD)
#define  nmDacPin    0x0F  // bitmask (1 of 8 set)
//unused registers used for accumulator and output pin mask
#define  nmDacAccu     0x0A
//nmDacOutMask  0x06 //long-aligned output pin mask
  
//file stamp\core\PWM.java (offsets within a VP bank) 
#define  nmPWMPort      0x0E  // 0 for PortA, 1 for PortB (reserved: 2 for PortC, 3 for PortD)
#define  nmPWMPin       0x0F  // bitmask (1 of 8 set)
//unused registers used for count registers and output pin mask
#define  nmPWMhighTime  0x02
#define  nmPWMlowTime   0x03
#define  nmPWMhighZero  0x04
#define  nmPWMlowZero   0x05
//nmPWMoutMask   0x06 //long-aligned output pin mask 
#define  nmPWMCount     0x0A
#define  nmPWMState     0x0B
  
//file stamp\core\Uart.java (offsets within a VP bank) 
#define  nmUartDataH     0x02  // transmit register highbyte
#define  nmUartDataL     0x03  // transmit register lowbyte 
#define  nmUartInvert    0x05  // 0 for normal, 1 for invert
#define  nmUartBaud      0x06  // 1000000/(bps * 4.34)
#define  nmUartStopBits  0x07  // Tx only: 00000001 for 1 stop bit, 00000011 for 2 etc
#define  nmUartPort      0x0C  // 0 for PortA, 1 for PortB (reserved: 2 for PortC, 3 for PortD)
#define  nmUartPin       0x0D  // bit mask (1 of 8 set)
#define  nmUartHSPort    0x0E  // 0 for PortA, 1 for PortB (reserved: 2 for PortC, 3 for PortD)
#define  nmUartHSPin     0x0F  // bitmask (1 of 8 set)
#define  nmUartBufferH   0x0A  // object reference of 256 byte buffer (so buffer starts at offset +4) 
#define  nmUartBufferL   0x0B
#define  nmUartHead      0x08  //Index of the next free byte in the buffer. 
#define  nmUartTail      0x09  //Index of the oldest byte in the buffer.
                               //If tailPointer  headPointer the buffer is empty.
//unused registers for count and divide
#define  nmUartCount     0x01
#define  nmUartDivide    0x04
  
//Uart baud constants (divide values)
#define  speed600    192  //The speed value for 600 baud.
#define  speed1200   96   //The speed value for 1200 baud.
#define  speed2400   48   //The speed value for 2400 baud.
#define  speed4800   24   //The speed value for 4800 baud.
#define  speed7200   16   //The speed value for 7200 baud.
#define  speed9600   12   //The speed value for 9600 baud.
#define  speed14400  8    //The speed value for 14400 baud.
#define  speed19200  6    //The speed value for 19200 baud.
#define  speed28800  4    //The speed value for 28800 baud.
#define  speed38400  3    //The speed value for 38400 baud.
#define  speed57600  2    //The speed value for 57600 baud.

#endif