/*
**********************************
* jvmEngine - executes bytecodes *
**********************************
*/

#include "jvmComm.h"
#include "jvmData.h"
#include "jvmStack.h"
#include "jvmConst.h"
#include "jvmLoad.h"
#include "jvmStore.h"
#include "jvmCompareJump.h"
#include "jvmInvoke.h"
#include "jvmMath.h"
#include "jvmArray.h"
#include "jvmObject.h"
#include "jvmSubReturn.h"
#include "jvmLogic.h"
#include "jvmConvert.h"
#include "jvmException.h"
#include "jvmDefines.h"
#include "jvmEngine.h"
  
   
long doStep(void) {
  long jemcode;
  //{{ perform a step }}
  jemcode = readJavaByte(getJvmPC());
  if (jemcode < Jem_INSTANCEOF) {
    switch (jemcode) {
      case Jem_NOP:             jb_nop();
	                            break;
      case Jem_ICONST_M1:       jb_iconst_m1();
	                            break;
      case Jem_ICONST_0:        jb_iconst_0();
	                            break;
      case Jem_ICONST_1:        jb_iconst_1();
	                            break;
      case Jem_ICONST_2:        jb_iconst_2();
	                            break;
      case Jem_ICONST_3:        jb_iconst_3();
	                            break;
      case Jem_ICONST_4:        jb_iconst_4();
	                            break;
      case Jem_ICONST_5:        jb_iconst_5();
	                            break;
      case Jem_BIPUSH:          jb_bipush();
	                            break;
      case Jem_SIPUSH:          jb_sipush();
	                            break;
      case Jem_ILOAD:           jb_iload();
	                            break;
      case Jem_ILOAD_0:         jb_iload_0();
	                            break;
      case Jem_ILOAD_1:         jb_iload_1();
	                            break;
      case Jem_ILOAD_2:         jb_iload_2();
	                            break;
      case Jem_ILOAD_3:         jb_iload_3();
	                            break;
      case Jem_ISTORE:          jb_istore();
	                            break;
      case Jem_ISTORE_0:        jb_istore_0();
	                            break;
      case Jem_ISTORE_1:        jb_istore_1();
	                            break;
      case Jem_ISTORE_2:        jb_istore_2();
	                            break;
      case Jem_ISTORE_3:        jb_istore_3();
	                            break;
      case Jem_IFEQ:            jb_ifeq();
	                            break;
      case Jem_IFNE:            jb_ifne();
	                            break;
      case Jem_IFLT:            jb_iflt();
	                            break;
      case Jem_IFLE:            jb_ifle();
	                            break;
      case Jem_IFGT:            jb_ifgt();
	                            break;
      case Jem_IFGE:            jb_ifge();
	                            break;
      case Jem_IF_ICMPEQ:       jb_if_icmpeq();
	                            break;
      case Jem_IF_ICMPNE:       jb_if_icmpne();
	                            break;
      case Jem_IF_ICMPLT:       jb_if_icmplt();
	                            break;
      case Jem_IF_ICMPGE:       jb_if_icmpge();
	                            break;
      case Jem_IF_ICMPGT:       jb_if_icmpgt();
	                            break;
      case Jem_IF_ICMPLE:       jb_if_icmple();
	                            break;
      case Jem_IF_ACMPEQ:       jb_if_acmpeq();
	                            break;
      case Jem_IF_ACMPNE:       jb_if_acmpne();
	                            break;
      case Jem_IRETURN:         jb_ireturn();
	                            break;
      case Jem_RETURN:          jb_return();
	                            break;
      case Jem_INVOKEVIRTUAL:   jb_invokevirtual();
	                            break;
      case Jem_INVOKESTATIC:    jb_invokestatic();
	                            break;
      case Jem_GETSTATIC:       jb_getstatic();
	                            break;
      case Jem_PUTSTATIC:       jb_putstatic();
	                            break;
      case Jem_POP:             jb_pop();
	                            break;
      case Jem_POP2:            jb_pop2();
	                            break;
      case Jem_DUP:             jb_dup();
	                            break;
      case Jem_DUP_X1:          jb_dup_x1();
	                            break;
      case Jem_DUP_X2:          jb_dup_x2();
	                            break;
      case Jem_DUP2:            jb_dup2();
	                            break;
      case Jem_DUP2_X1:         jb_dup2_x1();
	                            break;
      case Jem_DUP2_X2:         jb_dup2_x2();
	                            break;
      case Jem_SWAP:            jb_swap();
	                            break;
      case Jem_IADD:            jb_iadd();
	                            break;
      case Jem_ISUB:            jb_isub();
	                            break;
      case Jem_INEG:            jb_ineg();
	                            break;
      case Jem_IOR:             jb_ior();
	                            break;
      case Jem_ISHL:            jb_ishl();
	                            break;
      case Jem_ISHR:            jb_ishr();
	                            break;
      case Jem_IUSHR:           jb_iushr();
	                            break;
      case Jem_IAND:            jb_iand();
	                            break;
      case Jem_IXOR:            jb_ixor();
	                            break;
      case Jem_I2B:             jb_i2b();
	                            break;
      case Jem_GOTO:            jb_goto();
	                            break;
      case Jem_INVOKEINTERFACE: //not used by linker
	                            break;
      case Jem_ARRAYLENGTH:     jb_arraylength();
	                            break;
      case Jem_ATHROW:          jb_athrow();
	                            break;
      case Jem_CHECKCAST:       jb_checkcast();
	                            break;
      //default:
	}
  }
  else {
    switch (jemcode) {
      case Jem_INSTANCEOF:      jb_instanceof();
	                            break;
      case Jem_WIDE:            //not used by linker
	                            break;
      case Jem_IINC:            jb_iinc();
	                            break;
      case Jem_GETFIELD:        jb_getfield();
	                            break;
      case Jem_PUTFIELD:        jb_putfield();
	                            break;
      case Jem_NEW:             jb_new();
	                            break;
      case Jem_NEWARRAY:        jb_newarray();
	                            break;
      case Jem_ANEWARRAY:       jb_anewarray();
	                            break;
      case Jem_IALOAD:          jb_iaload();
	                            break;
      case Jem_IASTORE:         jb_iastore();
	                            break;
      case Jem_BALOAD:          jb_baload();
	                            break;
      case Jem_BASTORE:         jb_bastore();
	                            break;
      case Jem_LDC:             jb_ldc();
	                            break;
      case Jem_LDC_W:           jb_ldc_w();
	                            break;
      case Jem_LDC_W2:          //not used by linker
	                            break;
      case Jem_IMUL:            jb_imul();
	                            break;
      case Jem_IDIV:            jb_idiv();
	                            break;
      case Jem_IREM:            jb_irem();
	                            break;
      case Jem_JSR:             //not used by linker
	                            break;
      case Jem_RET:             //not used by linker
	                            break;
      case Jem_TABLESWITCH:     jb_tableswitch();
	                            break;
      case Jem_LOOKUPSWITCH:    jb_lookupswitch();
	                            break;
      //default:
	}
  }
  return -1; //no return in spin code
}

long doBytecode(void) {
  long state,c;
  /* Execute a bytecode */
  if ((getJvmPC() < 0x0014) && (getActivity() != kEnd)) { //0x0014 is address of first class header
    setActivity(kEnd);
    end();
  }
  state = getActivity();
  switch (state) {
    case kRun    : //JVM running
                   if (isBreakpoint()) {
                     breakpoint();
                     return -1;
                   }
				   else {
                     c = doStep();
                     if (mustNotifyStepDone())
                       sendStepDone();
                     return c;
				   }
				   break;
    case kBreak  : //appears not to be used
                   //so change state to kPassive
                   setActivity(kPassive);
                   return -1;
    case kPassive: //JVM was running but has stopped (exception,stopped,end)
                   if (mustStep()) {
                     c = doStep();
                     if (mustNotifyStepDone()) {
                       sendStepDone();
					 }
					 return c;
				   }
                   return -1;
    case kJog    : //appears not to be used
                   //so change state to kPassive
                   setActivity(kPassive);
                   return -1; 
    case kEnd    : //do nothing
                   return -1;
	//default:
  }  
  return -1;
}

          
      